<?php
// Autor Miroslav Novak www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// $Id: MuzoWebServices.php,v 1.6 2011/01/11 06:52:08 mira Exp $

require_once('muzo.php');


class MuzoWebServices {

	/** @var string */
	public $MerchantNumber;
	/** @var string */
	protected $MuzoPublicKeyFile;
	/** @var string */
	protected $PrivateKeyFile;
	/** @var string */
	protected $PrivateKeyPass;
	/** @var SoapClient */
	public $mws;
	/** @var string */
	protected $provider;
	/** @var UniLogger */
	protected $logger;

	/**
	 * @param string $wsdl
	 * @param string $serviceUrl
	 * @param string $provider
	 * @param string $merchantNumber
	 * @param string $muzoPublicKeyFile
	 * @param string $privateKeyFile
	 * @param string $privateKeyPass
	 * @param UniLogger $logger
	 * @return void
	 */
	function __construct(
		$wsdl,                    // soubor popisu webservice
		$serviceUrl,              // url webservice
		$provider,         	      // provider, typicky kod banky
		$merchantNumber,          // cislo obchodnika
		$muzoPublicKeyFile,       // soubor s verejnym klicem Muzo
		$privateKeyFile,          // soubor s privatnim klicem
		$privateKeyPass,          // heslo privatniho klice
		$logger)
	{
		$this->logger = $logger;
		$this->MerchantNumber = $merchantNumber;
		$this->MuzoPublicKeyFile = $muzoPublicKeyFile;
		$this->PrivateKeyFile = $privateKeyFile;
		$this->PrivateKeyPass = $privateKeyPass;
		$this->provider = $provider;
		$this->mws = new SoapClient($wsdl, ['cache_wsdl'=>WSDL_CACHE_NONE, 'trace'=>true, 'exceptions'=>true, 'location'=>$serviceUrl  ]);
	}

	// nasleduji jednotlive funkce poskytovane pres Muzo Webservices
	// navratova hodnota urcuje zda byl podpis muzo ok (true, false)
	// posledni argument $res je pole obsahujici navratove hodnoty funkce WebServices
	// popis jednotlivych funkci je uveden v souboru "PayMuzo - Web Services.pdf" dokumentace dodane od Muzo
	// chybu pri komunikaci se WebServices lze zjistit volanim funkce GetError, ktera pri chybe vrati neprazdny retezec s popisem chyby

	/**
	 * @param string $orderNumber
	 * @return object
	 */
	function getPaymentStatus($orderNumber) {
		return $this->callServiceSig('getPaymentStatus','paymentStatus',array('paymentNumber'=>$orderNumber));
	}

	/**
	 * @param mixed $orderNumber
	 * @return object
	 */
	function getPaymentDetail($orderNumber) {
		return $this->callServiceSig('getPaymentDetail','paymentDetail',array('paymentNumber'=>$orderNumber));
	}

	/**
	 * @param string $paymentNumber
	 * @param string $masterPaymentNumber
	 * @param string $orderNumber
	 * @param string $referenceNumber
	 * @param float $amount
	 * @param string $currencyCode
	 * @param bool $captureFlag
	 * @return object
	 */
	function processRecurringPayment(
			$paymentNumber, //orderNumberarray
			$masterPaymentNumber,
			$orderNumber, //merOrdNumarray
			$referenceNumber, //md
			$amount,
			$currencyCode,
			$captureFlag
		) {
			return $this->callServiceSig('processRecurringPayment', 'recurringPayment',array(
				'paymentNumber'=>$paymentNumber,
				'masterPaymentNumber'=>$masterPaymentNumber,
				'orderNumber'=>$orderNumber,
				'referenceNumber'=>$referenceNumber,
				'amount'=>$amount,
				'currencyCode'=>$currencyCode,
				'captureFlag'=>$captureFlag
			));
	}


	/**
	 * @param string $paymentNumber
	 * @param string $masterPaymentNumber
	 * @param string $orderNumber
	 * @param string $referenceNumber
	 * @param float $amount
	 * @param string $currencyCode
	 * @param bool $captureFlag
	 * @return object
	 */
	function processUsageBasedSubscriptionPayment(
			$paymentNumber,
			$masterPaymentNumber,
			$orderNumber,
			$referenceNumber, //md
			$amount,
			$currencyCode,
			$captureFlag
		) {
			return $this->callServiceSig('processUsageBasedSubscriptionPayment', 'usageBasedSubscriptionPayment',array(
				'paymentNumber'=>$paymentNumber,
				'masterPaymentNumber'=>$masterPaymentNumber,
				'orderNumber'=>$orderNumber,
				'referenceNumber'=>$referenceNumber,
				'amount'=>$amount,
				'currencyCode'=>$currencyCode,
				'captureFlag'=>$captureFlag
			));
	}

	/**
	 * @param string $paymentNumber
	 * @param float $amount
	 * @return object
	 */
	function processCapture(
			$paymentNumber,
			$amount
		) {
		return $this->callServiceSig('processCapture', 'capture', array(
			'paymentNumber'=>$paymentNumber,
			'amount'=>$amount
		));
	}

	/**
	 * @param string $paymentNumber
	 * @param int $captureNumber
	 * @return object
	 */
	function processCaptureReverse(
			$paymentNumber,
			$captureNumber // !!!! not quite sure what this is, currently just putting 1
		) {
		return $this->callServiceSig('processCaptureReverse', 'captureReverse', array(
			'paymentNumber'=>$paymentNumber,
			'captureNumber'=>$captureNumber
		));
	}

	/**
	 * @return object
	 */
	function processBatchClose() {
		return $this->callServiceSig('processBatchClose', 'batchClose', array(), $requestAppend="");
	}

	/**
	 * @param string $paymentNumber
	 * @return object
	 */
	function processAuthorizationReverse(
			$paymentNumber
		) {
		return $this->callServiceSig('processAuthorizationReverse', 'authorizationReverse', array(
			'paymentNumber'=>$paymentNumber
		));
	}

	/**
	 * @param string $paymentNumber
	 * @param float $amount
	 * @return object
	 */
	function processRefund(
			$paymentNumber,
			$amount
		) {
		return $this->callServiceSig('processRefund', 'refund', array(
			'paymentNumber'=>$paymentNumber,
			'amount'=>$amount
		), "Request", "RequestResponse");
	}



	/**
	 * @param string $funcName
	 * @param string $parNameBase
	 * @param array<string, mixed> $params
	 * @param string $requestAppend
	 * @param string $responseAppend
	 * @return object
	 * @throws SoapFault
	 * @throws Exception
	 */
	function callServiceSig($funcName, $parNameBase, $params, $requestAppend="Request", $responseAppend="Response") {
		$messageId = uniqid('xxxxx');
		$params = array('messageId'=>$messageId, 'provider'=>$this->provider, 'merchantNumber'=>$this->MerchantNumber) + $params;
		$sigstr = implode('|', $params);
		$sig = muzo_Sign($sigstr, $this->PrivateKeyFile, $this->PrivateKeyPass);
		$params['signature'] = base64_decode($sig);
		$params = array($parNameBase.$requestAppend => $params);
		try {
			$paramsLog = $params; unset($paramsLog[$parNameBase.$requestAppend]['signature']);
			$this->logger->writeLogNoNewLines("GPWP SOAP request: " . $funcName . ' ' . print_r($paramsLog, true));
			$res = call_user_func([$this->mws, $funcName], $params);
		} catch(SoapFault $e) {
			//$this->logger->writeLog("DEBUG GPWebPay: callServiceSig exception: ".var_export($e, true)); // debug
			$eLog = $e; // previously was '$eLog = unserialize(serialize($e))' but threw "Serialization of 'Closure' is not allowed" exception
			unset($eLog->detail->paymentServiceException->signature); unset($eLog->detail->serviceException->signature);
			$this->logger->writeLogNoNewLines("GPWP SOAP ERROR exceptionMessage: '" . print_r($eLog->faultstring, true) . "' exceptionDetail: " . print_r($eLog->detail, true));
			throw $e;
		}
		$eRes = unserialize(serialize($res)); unset($eRes->{$parNameBase.$responseAppend}->signature);
		$this->logger->writeLogNoNewLines("GPWP SOAP response: " . $funcName . ' ' . print_r($eRes, true));
		$res = $res->{$parNameBase.$responseAppend};
		$sigr = $res->signature;
		$resar = get_object_vars($res);
		unset($resar['signature']);
		$sigrstr = implode('|', $resar);
		if (!(muzo_Verify($sigrstr, base64_encode($sigr), $this->MuzoPublicKeyFile))) {
			$this->logger->writeLogNoNewLines("GPWP SOAP response validation error: neplatny podpis nebo messageId");
			throw new Exception("GPWP SOAP response validation error: neplatny podpis nebo messageId");
		}
		return $res;
	}

}

