<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV098-49-ga9cb64a5:2022-11-14#

namespace UniModul\Base;

interface SessionInterface {

	/**
	 * @param string $name
	 * @param mixed $default
	 * @return mixed
	 */
	public function get($name, $default = null);

	/**
	 * @param string $name
	 * @param mixed $value
	 * @return void
	 */
	public function set($name, $value);

	/**
	 * @param string $name
	 * @return bool
	 */
	public function has($name);

	/**
	 * @param string $name
	 * @return void
	 */
	public function remove($name);

}

class StandardSession implements SessionInterface {

	public function get($name, $default = null) {
		$this->init();
		return isset($_SESSION[$name]) ? $_SESSION[$name] : $default;
	}

	public function set($name, $value) {
		$this->init();
		$_SESSION[$name] = $value;
	}

	public function has($offset) {
		$this->init();
		return isset($_SESSION[$offset]);
	}

	public function remove($name) {
		$this->init();
		unset($_SESSION[$name]);
	}

	/** @return void  */
	public function init() {
		if (session_status() == PHP_SESSION_NONE) {
			session_start();
		}
	}
}

